/***************************************************************************
  DJE 9/23/24
    OLED using u8g2, bmp280 Pressure and humidity. 
    On Adafruit Itsy Bitsy MO Express, SAMD21
    bmp280 barometric sensor:
      Chinese bmp280: +3.3, GND, SCL, SDA, n/c, n/c  
      Need to change to BMP280_ADDRESS_ALT I2C Addr: 0x76, 
      or drive pin 6 to HI I2C Addr: 0x76 
      OLED: 2.4" SPI
      Use HW SPI, full buffer
      Needed #include "avr/dtostrf.h" to get float to string working
  DJE 9/26/24 
    Added 1mS Timer interrupt SAMD from  Khoi Hoang. Works well. \
  DJE 10/9/24 Tested to see if SPI and Bit bang can share pins. No problem. 
    Just set pins, then run SPI.Begin() after GPIO. 
    But plan to use SPI for 
      OLED for now, not with Nextion
      AD55522 Quad SMU, Needs one CS/
      AD7685 ADC. OK to share SDO (MISO). No problem to tie together
  DJE 10/12/24 
    Setting Voltages working.
      Had to remove LVDS jumpers. Board CPU worked with them, not with Arduino
      SPI MODE1: Positive clock latch data on neg edge. Works OK with OLED too. Maybe set in setup()??
  DJE 10/15/24    
    Commented out annoying #warning messages in SAMD timer library
  DJE 10/18/24
    Read function working for DAC registers. Decided not to use it, too slow
  DJE 10/23/24
    Wrote first Set function for I range using global shadows for PMU[]. 
    Tested all I ranges. Works swell. 
  DJE 11/5/24
    Controls for adjusting VF cal working
    EEPROM functions, but no EEPROM on SAMD21! EFlash emulation erases on programming!
    Change to Teensy $24 or something. Or add I2C EEMPRM AT24C08 $.50
  DJE 11/6/24 
    IF is working, hand-cal 1 channel. Need function for force dac 
  DJE 11/11/24 
    Measure is working. +5V was set low, causing errors. 
    Moving measure and cal to AD5522.ino
  DJE 11/15/24 Backup. 
    Updated measure to incorporate cal, and moved in-line code to function.
    Current cal on 2mA works OK on other ranges too. 
    Display current scaling and formatting working across all ranges. 
    Aded analog encoder
  DJE 1/9/25 
    New board in, OLED, power, CPU working. No AD5522 yet. 
  DJE 1/12/25
    Keys are scanning, encoder working.
  DJE 1/13/25
    Display and cursor structs


Remaining firmware tasks:
*  Change Rsh for 20mA (40 ohms, 25mA), and add comp caps to CH1, CH3
  Measure scaling and cal for all ranges
  Cal using external EEPROM
  Front panel board: 
    Encoder,  Buttons, LEDs
  Multiple screens


 ***************************************************************************/

#include <Arduino.h>
#include <U8g2lib.h>
#include <stdio.h>
#include <math.h>
#include "avr/dtostrf.h"    // needed on M0 for sprintf and dtostrf()
#include "AD5522.h"
#include "display.h"
//#include <digitalWriteFast.h> 

#include <Wire.h>
#include <SPI.h>

// Timer Setup: Select timer for SAMD21. Must must be placed at the beginning before #include "SAMDTimerInterrupt.h"
#define USING_TIMER_TC3   true      // Only TC3 can be used for SAMD51 can use TC3, TC4, TC5, TCC, TCC1, TCC2
#include "SAMDTimerInterrupt.h"
// TC3, TC4, TC5 max permissible TIMER_INTERVAL_MS is 1398.101 ms, larger will overflow, therefore not permitted
// Use TCC, TCC1, TCC2 for longer TIMER_INTERVAL_MS
#define TIMER_INTERVAL_MS      1
#define SELECTED_TIMER      TIMER_TC3 // Change for other timers
// Init selected SAMD timer
SAMDTimer ITimer(SELECTED_TIMER);

#ifndef LED_BUILTIN
#define LED_BUILTIN   13
#endif
// Control pins
#define C0 14
#define C1 15
#define R0 16
#define R1 17
#define R2 18
#define R3 19

// Constructor for large 2.4" OLED, SSD1309, SPI
U8G2_SSD1309_128X64_NONAME0_F_4W_HW_SPI u8g2(U8G2_R0, /* cs=*/ 5, /* dc=*/ 7, /* reset=*/ 9);  // For SAMD21

/****************** Globals *******************************/
int ch;       // This is the active channel
unsigned short calC = 0x8000;     // Initial Cal values
unsigned short calM = 0xFFFF;

char command;        // For encoder
int encCount = 0;
int oldEncCount = 0;

char keyFlag[16];     // Keyboard key down flags

int menu = 0;       // 0: Main (Force), 1: Range, 2: Mode, 3: Clamps
int memuMode = 0;   // 0: Main, 1: select
int menuMax = 3;
/******************** setup() ****************************/

void setup() {
  int i, j, range;

  u8g2.begin();
  dispInit();
  Serial.begin(115200);
//  analogReadResolution(12);

  pinMode(LED_BUILTIN, OUTPUT);

  pinMode(AD5522_CS, OUTPUT);
  pinMode(ADC_CNV, OUTPUT);

  pinMode(C0, OUTPUT);
  pinMode(C1, OUTPUT);
  digitalWrite( C0, 0);
  digitalWrite( C1, 0);
  pinMode(R0, INPUT_PULLUP);
  pinMode(R1, INPUT_PULLUP);
  pinMode(R2, INPUT_PULLUP);
  pinMode(R3, INPUT_PULLUP);

  digitalWrite(AD5522_CS, HIGH);
  digitalWrite(ADC_CNV, LOW);



  //while ( !Serial ) delay(100);   // wait for native usb. Hangs if no serial port open
  delay (1000);

  unsigned status;

  // Timer Init
  status = ITimer.attachInterruptInterval_MS(TIMER_INTERVAL_MS, TimerHandler);
  if (status) Serial.println("Initializing timer"); 
  else Serial.println("Timer error"); 

  AD5522Init();
  
  //Set force value cursor to Units
 
  ch = 0;
 
//#define BOARD_PROTO
#define BOARD_1
 // Proto
#ifdef BOARD_PROTO
  Fcal[0][VF].C = 0x7FE8;  // Channel O VF Cal
  Fcal[0][VF].M = 0xFFF0; 
  Fcal[1][VF].C = 0x7FF5;  // Channel 1 VF Cal
  Fcal[1][VF].M = 0xFFFD; 
  Fcal[2][VF].C = 0x7FE8;  // Channel 2 VF Cal
  Fcal[2][VF].M = 0xFFF0; 
  Fcal[3][VF].C = 0x7FF3;  // Channel 3 VF Cal
  Fcal[3][VF].M = 0xFFFF; 
#endif
  // Board 1, using new scaling
#ifdef BOARD_1  
  Fcal[0][VF].C = 0x800B;  // Channel O VF Cal
  Fcal[0][VF].M = 0xFFE8; 
  Fcal[1][VF].C = 0x7FFA;  // Channel 1 VF Cal
  Fcal[1][VF].M = 0xFFF1; 
  Fcal[2][VF].C = 0x8017;  // Channel 2 VF Cal
  Fcal[2][VF].M = 0xFFE5; 
  Fcal[3][VF].C = 0x7FF7;  // Channel 3 VF Cal
  Fcal[3][VF].M = 0xFFF7; 
  for( ch = 0; ch <= 3; ch++){ 
    AD5522SetVoltCal(ch, Chan[ch].forceV);
  }
  ch = 0;
  
  disp[1].cursor = 3;
#endif

  //Test current force 
  AD5522SetFMode(0, FI);
  AD5522SetFMode(1, FI);
  AD5522SetFMode(2, FI);
  AD5522SetCurr(0, 0.002);
  AD5522SetCurr(1, 0.001);
  AD5522SetCurr(2, -0.002);

  
 // eeWriteCalForce(0, VF, 1234, 5678 );


} // end of setup()

/******************** loop() ****************************/

void loop() {
  long timer;
  unsigned long ms;
  int i;
  long adc = 0;

//  unsigned long spiRData = 0l;
  float meas;   // temp variable
  long adc2; 
  float DACV;
  unsigned long dacVal;
  unsigned long DACData; 
  
  getCommand();

  
  // AD5522SetIRange(0, IR2MA);
  // AD5522SetIRange(3, IR50MA);
  // AD5522SetVoltCal(ch, Chan[ch].forceV);



  // printCal(0);
  // printCal(1);
  // printCal(2);

  // Measure V and I 
  setMeasure(ch, MV);
  ad5522Meas(ch, 0);
  setMeasure(ch, MI); 
  ad5522Meas(ch, 0);

  // printPMU();
  // ad5522RD(0, AD55_MCR);
  
  drawScreen1();        // Display measure values
  // delay between each page
  // delay(100);

}
