/****************** Functions ****************************/
void TimerHandler()
{
  //timer interrupt toggles pin LED_BUILTIN
  digitalWrite(LED_BUILTIN, 1);

//  readEncoder();
  readButtons();
  digitalWrite(LED_BUILTIN, 0);
}

/* Reads analog encoder. A has 1.0K, B has 2.0K, pull-up is 5K, cap is .1uF
  Works with encoders that are 4 states per click: TT or bourns
  OFF - A - Both - B  0-1-2-3
  OFF - B - Both - A  0-3-2-1
  Idle, 0, both OFF : 3K / (5K + 3K) = 3/8 * 1023 = 383
  A,  1 ON          : 2K / (5K + 2K) = 2/7 * 1023 = 299
  B, 3 ON           : 1K / (5K + 1K) = 1/6 * 1023 = 170
  Both ON, 2        : 0K / 5K + 0K)  = 0/5 * 1023 = 0
  Tolerance of +/- 30 counts seems to work well. 
  Reads ADC once, checks against limits, assigns state. 
  Then encCount detects transitions from Both to A  (increment)
  or Both to B (decrement). 
  To change direction, change encCount ++ to -- and vice versa
  This is annoying and complicated. And it only saves one wire. 
  Bad idea. 
*/
void readAnEncoder(void){   
  #define EO 383
  #define EA 299
  #define EB 170
  #define EAB 0 
  static char encSt, encStD1, encStD2; // idle. 1:A, 2:BOTH, 3: B

  #define tol 30    // +/- Tolerance (range for valid encoder ADC values
  
  int encAn1 = analogRead(0);
  int encAn = analogRead(0);
  
  if ((encAn - encAn1) < 20 && (encAn1 - encAn) < 20){    // Read twice, check for stable V 
    //Serial.println( encAn);
    if (encAn > (EO-tol) && encAn < (EO+tol)) {     // Idle
      encSt = 0;    
    }
    if (encAn > (EA-tol) && encAn < (EA+tol)) {     // A
      encSt = 1; 
 //     Serial.print("1a ");
    }   
    if (encAn > (EB-tol) && encAn < (EB+tol))  {    // B
      encSt = 3; 
//      Serial.print("3b ");    
    }  
    if (encAn < (EAB+tol)) {                        // Both     
      encSt = 2; 
 //     Serial.print("2B ");    
    }                     
    if (encStD1 == 2 && encSt == 1) {      // Transition BOTH to A
      encCount --;
 //     Serial.print( "encCount = "); Serial.println( encCount);
    }
    if (encStD1 == 2 && encSt == 3) {
      encCount++;
 //     Serial.print( "encCount = "); Serial.println( encCount);
    }
    encStD2 = encStD1;
    encStD1 = encSt;
  }

}

/* Read the buttons and the encoder. 
  Scan the columns, read the rows. Increment key state[] count for debounce. 
  Takes about 42uS. Would be faster with faster reads and writes. 
  Could be a FOR loop
  */

#define KEY_COUNT 16
#define KEY_DEB 9

void readButtons(void){ 
  static char state[16];
  static char encSt, encStD1; // 0:idle. 1:A, 2:BOTH, 3: B
  char row, col, i; 
  
  digitalWrite(C1, 0); digitalWrite (C0, 0);
  encSt = (digitalRead(R1) << 1) | digitalRead(R0) ;  // Read Encoder A and B into encSt
  
  if(digitalRead(R2)==0) state[2] ++; else state[2] = 0;
  if(digitalRead(R3)==0) state[3] ++; else state[3] = 0;

  digitalWrite(C1, 0); digitalWrite (C0, 1);
  if(digitalRead(R0)==0) state[4] ++; else state[4] = 0;
  if(digitalRead(R1)==0) state[5] ++; else state[5] = 0;
  if(digitalRead(R2)==0) state[6] ++; else state[6] = 0;
  if(digitalRead(R3)==0) state[7] ++; else state[7] = 0;

  digitalWrite(C1, 1); digitalWrite (C0, 0);
  if(digitalRead(R0)==0) state[8] ++; else state[8] = 0;
  if(digitalRead(R1)==0) state[9] ++; else state[9] = 0;
  if(digitalRead(R2)==0) state[10] ++; else state[10] = 0;
  if(digitalRead(R3)==0) state[11] ++; else state[11] = 0;

  digitalWrite(C1, 1); digitalWrite (C0, 1);
  if(digitalRead(R0)==0) state[12] ++; else state[12] = 0;
  if(digitalRead(R1)==0) state[13] ++; else state[13] = 0;
  if(digitalRead(R2)==0) state[14] ++; else state[14] = 0;
  if(digitalRead(R3)==0) state[15] ++; else state[15] = 0;
  
  // Debounce counters
  for (i=0; i<KEY_COUNT; i++) {                     
    if (state[i] == KEY_DEB) (keyFlag[i] = 1);
    if (state[i] > KEY_DEB) state[i] = 10;
  }

  digitalWrite(C1, 0); digitalWrite (C0, 0);    // Set Col to 0 on exit

  // Encoder processing
  if (encStD1 == 0 && encSt == 1) {      // Transition BOTH to A
    encCount --;
  }
  if (encStD1 == 0 && encSt == 2) {
    encCount++;
  }
  encStD1 = encSt;

}

