#include "display.h"

// Set up display structs initial values. Must be called by setup()
void dispInit(void) {
  // Measure V
  disp[0]  = {11.0, -11.0, 10.0, 0 , NUM1, 90, 0, 0, (char*)"Vm", (char*)"V" };
  // Force V
  disp[1]  = {11.0, -11.0, 10.0, 0 , 0, 66, 0, 00, (char*)"Vf", (char*)"V" };
  // Measure I
  disp[2]  = {11.0, -11.0, 10.0, 0 , NUM2, 90, 0, 0, (char*)"Im", (char*)"mA" };
  // Force I 5mA
  disp[4]  = {6.0, -6.0, 5.0, 0 , 0, 66, 0, 00, (char*)"If", (char*)"uA" };
  // Force I 20uA
  disp[5]  = {24.0, -24.0, 20.0, 0 , 0, 66, 0, 00, (char*)"If", (char*)"uA" };
  // Force I 200uA
  disp[6]  = {2.4, -2.4, 2.0, 0 , 0, 66, 0, 00, (char*)"If", (char*)"uA" };
  // Force I 2mA
  disp[7]  = {2.4, -2.4, 2.0, 0 , 0, 66, 0, 00, (char*)"If", (char*)"mA" };
  // Force I 25mA
  disp[8]  = {30, -30, 25.0, 0 , 0, 66, 0, 00, (char*)"If", (char*)"mA" };
}

void drawValue(int t){        // Display Vm, no set (cursor)
  char buf1[32];

  // Should check range of t
  // Draw Label
  u8g2.drawStr( disp[t].xPos, disp[t].yPos, disp[t].label );
  // Draw Value
  dtostrf(Chan[ch].measV, 7, 3, buf1);
  u8g2.drawStr( disp[t].xPos + 20, disp[t].yPos, buf1 );
  // Draw Units
  u8g2.drawStr( disp[t].xPos + disp[t].uPos, disp[t].yPos, disp[t].units );
}

void drawSetValue(int t){        // Display Vf with cursor. t is type
  char buf1[32];
  unsigned char cursorSta, cursorEnd;
  // Should check range of t
  // Draw Label
  u8g2.drawStr( disp[t].xPos, disp[t].yPos, disp[t].label );
  // Draw Value. Need to switch on V and I ranges
  dtostrf(Chan[ch].forceV, 7, 3, buf1);
  u8g2.drawStr( disp[t].xPos + 18, disp[t].yPos, buf1);
  // Draw Units
  u8g2.drawStr( disp[t].xPos + disp[t].uPos, disp[t].yPos, disp[t].units );
  // cursor calc: 6 pix per char, 9 characters
  if (menu == 0){   // Force Menu, underline force
    cursorSta = (9 - disp[t].cursor) * 6;
    if (disp[t].cursor > 2) cursorSta -= 6; // DP correction
    cursorEnd = cursorSta + 6;
    // Vertical offset for cursor
    u8g2.drawLine(cursorSta, disp[t].yPos + 9, cursorEnd, disp[t].yPos + 9);
  }

}

void u8g2_prepare(void) {
  u8g2.setFont(u8g2_font_6x10_tf);
  u8g2.setFontRefHeightExtendedText();
  u8g2.setDrawColor(1);
  u8g2.setFontPosTop();
  u8g2.setFontDirection(0);
}



void drawScreen1(void){
  char buf1[32];
  unsigned int range;
  float iScale; 
  char iUnits[8];
  char digits;
  //itoa (atod, buf1, 10);      // Display Dac decimal

  u8g2.clearBuffer();
  u8g2_prepare();

  u8g2.setFont(u8g2_font_6x10_tf);
  drawSetValue(1);
  u8g2.setFont(u8g2_font_9x15_tf);
  
  drawValue(0);

  // dtostrf(Chan[ch].measV, 7, 3, buf1);
  // u8g2.drawStr( 0, NUM1, "Vm ");
  // u8g2.drawStr( 20, NUM1, buf1);
  // u8g2.drawStr( 90, NUM1, "V");

  // Display Im. Requires changing scaling and 
  range = Chan[ch].IRange;
  if (range == IR5UA){
    iScale = 1000000.0;
    digits = 3;
    u8g2.drawStr( 90, NUM2, "uA"); 
  }
  if (range == IR20UA){
    iScale = 1000000.0;
    digits = 3;
    u8g2.drawStr( 90, NUM2, "uA");
  }
  if (range == IR200UA){
    iScale = 1000000.0;
    digits = 2;
    u8g2.drawStr( 90, NUM2, "uA");
  }
  if (range == IR2MA){
    iScale = 1000.0;
    digits = 3;
    u8g2.drawStr( 90, NUM2, "mA");
  }
  if (range == IR50MA){
    iScale = 500.0;
    digits = 2;
    u8g2.drawStr( 90, NUM2, "mA");
  }
    // if (range ==  IR20UA) iScale = 0.00002;
    // if (range ==  IR200UA) iScale = 0.0002;
    // if (range ==  IR2MA) iScale = 0.002;
    // if (range ==  IR50MA) iScale = 0.05;
  dtostrf(Chan[ch].measI * iScale, 7, digits, buf1);
  u8g2.drawStr( 0, NUM2, "Im ");
  u8g2.drawStr( 20, NUM2, buf1);
  // u8g2.drawStr( 110, NUM2, "mA");

  // Draw channel, Small font for bottom line status
  u8g2.setFont(u8g2_font_6x10_tf);    
  dtostrf(ch, 2, 0, buf1);
  u8g2.drawStr( LEFT, BOTTOM, "Ch ");
  u8g2.drawStr( LEFT + 12, BOTTOM, buf1);
  //Draw force mode
  if (Chan[ch].FMode == FV) u8g2.drawStr( RIGHT, BOTTOM, "Force V");
  if (Chan[ch].FMode == FI) u8g2.drawStr( RIGHT, BOTTOM, "Force I");
  //Display current range
  if (range == IR5UA)    u8g2.drawStr(CENTER, BOTTOM, "IR 5uA"); 
  if (range == IR20UA)   u8g2.drawStr(CENTER, BOTTOM, "IR 20uA");
  if (range == IR200UA)  u8g2.drawStr(CENTER, BOTTOM, "IR 200uA");
  if (range == IR2MA)    u8g2.drawStr(CENTER, BOTTOM, "IR 2mA");
  if (range == IR50MA)   u8g2.drawStr(CENTER, BOTTOM, "IR 50mA");

  if (menu == 1){   // Range Menu, underline range
    u8g2.drawLine(CENTER, BOTTOM + 9 , CENTER + (6 * 6), BOTTOM + 9 );
  }
  if (menu == 2){   // Mode Menu, underline range
    u8g2.drawLine(RIGHT, BOTTOM + 9 , RIGHT + (4 * 6), BOTTOM + 9 );
  }

  u8g2.sendBuffer();
}

// Clamp Screen
void drawScreen2(void){

}
